var debug = 0;                 // 0 = no debug overly // 1 = debug overlay

var w = window.innerWidth;     // global reference width
var h = window.innerHeight;

var xBgndPosition   = .475;    // initial bgnd img position
var yBgndPosition   = .8;
var buttonX         = [   .1, .471,  .833,  .764  ]; // button coordinates
var buttonY         = [   .1, .391,  .370,  .575  ];
var buttonState     = [   -1,   -1,     -1,    -1  ]; // button states
var button          = new Array(buttonX.length);               // ID's of generated buttons
var buttonSize      = w/3;   // default value: will be updated in layout()
var imgScale        = 1.5; // determines the height of the bgnd image in multiple screen heights
var parallax        = 0;   // strength of parallax effect (0=nothing ... 1)
var xCenter         = 0;
var yCenter         = 0;
var imagesLoaded    = 0;   // flag: 1 if images are loaded
var timeCount       = 0;
var imgHeight       = 50;  // img size on screen default values: will be updated in layout()
var imgWidth        = 50;
var buttonsAreGenerated = 0; // flag: 1 if buttons are generated
var startResetTime  = 0;

var socket;


//============================================================== INIT
function init() {
  document.getElementById('bgndImage').ontouchstart = startTouch;
  document.getElementById('bgndImage').ontouchend   = endTouch;
  document.getElementById('interface').ontouchstart = startTouch;
  document.getElementById('interface').ontouchend   = endTouch;
  document.getElementById('interface').onclick = startTouch;
    //establish websocket connection and add callback functions
    socket = io.connect("http://smartfairytale:3000"); //TODO: change server name!!!! // !!
    socket.on('initMessage',receiveInit);   // !!
    socket.on('buttonMessage',receiveButton);   // !!
   

    // layout(); //moved to receiveInit()
    // startTimer(); // moved to receiveInit()
}


//============================================================== RECEIVE INIT
function receiveInit(data){
  
  
  buttonState = data.state; // !! 
  layout();
  startTimer(); 
  setButtonColors();

}


//============================================================== ANIMATION TIMER
function startTimer() {
  timeCount = 0;
  var id = setInterval(loop, 10);
  function loop() {
    showDebug();
    //----------------------------------------------------------------------------- LOOP BEGIN
    timeCount++;
    if (timeCount>startResetTime && buttonState[0]==1){                    // reset start button to red
      buttonState[0]=-1;
      setButtonColors();
    }


    if (document.getElementById('bgndImage').complete && document.getElementById('i1').complete
       && document.getElementById('i2').complete && document.getElementById('i3').complete
       && document.getElementById('i4').complete && imagesLoaded == 0){
        // do only once if images are loaded completely:
        imagesLoaded=1; // set loaded flag
        //----------------------------- bgnd image
        var imgOrigWidth  = document.getElementById('bgndImage').naturalWidth; // get original image dimensions in px
        var imgOrigHeight = document.getElementById('bgndImage').naturalHeight;
        xCenter           = (imgWidth - w)  * xBgndPosition;                                  // calculate scroll positions for centering image in x and y
        yCenter           = (imgHeight - h) * yBgndPosition;
        window.scrollTo(xCenter,yCenter);                                      // scroll to center
        document.getElementById('bgndImage').style.opacity=1;                  // fade in bgnd image
        document.getElementById('interface').style.opacity=1;                  // fade in binterface
    }
    //----------------------------------------------------------------------------- LOOP END
  }
}

//============================================================== TOUCH
function startTouch() { // function is called when touch starts
  console.log('touch starts');

}
function endTouch() { // function is called when touch ends
  console.log('touch ends');
  // buttonState[0]=-1;
  setButtonColors();
}

//============================================================== LAYOUT
function layout(){
  w = window.innerWidth;
  h = window.innerHeight;
  //----------------------------- position & size bgnd image
  var bgndImg = document.getElementById('bgndImage');

  var imgOrigWidth  = bgndImg.naturalWidth;            // get original image dimensions in px
  var imgOrigHeight = bgndImg.naturalHeight;
  var imgRatio      = imgOrigWidth / imgOrigHeight;    // get image ratio


  if(bgndImg && bgndImg.style) {                       // set image size
      imgHeight= w * 3 ;
      imgWidth = imgHeight * imgRatio;
      buttonSize      = w/3;                                    // size of buttons
      parallax =  w/5000; // strength of effect (0=nothing ... 1)
      if (w>h){
        imgHeight= w /imgRatio;
        imgWidth = w;
        buttonSize      = w/8;                                    // size of buttons
        parallax =  w/15000; // strength of effect (0=nothing ... 1)
      }

      bgndImg.style.height = imgHeight + 'px';
      bgndImg.style.width  = imgWidth + 'px';
  }

  //----------------------------- generate & style clickable button-dots in 'interface'-div
   generateButtons();           // can be called more than once as the interface is cleared before buttons are generated
   setButtonColors();

  //----------------------------- position & size debug box
  var infoBox = document.getElementById('infoBox');
  infoBox.style.width     =  w-w/25+'px'
  if (h>w){
    infoBox.style.height  =  w/3+'px'; // portrait
  }else{
    infoBox.style.height  =  h/5+'px'; // landscape
  }
  infoBox.style.left      =  w/50+'px';
  infoBox.style.top    =  w/50+'px';
  infoBox.style.padding   = '0px';
  infoBox.innerHTML       = "1";

}
//============================================================== GENERATE BUTTONS // DU_ROCKST!
function generateButtons(){
  if (buttonsAreGenerated==0){
    var interface = document.getElementById('interface');
    interface.innerHTML ='';                                     // clear interface div
    // generate start button (replaces array index 0)
    interface.innerHTML += '<div id="dot0" onclick="buttonClicked(0)"><img id="buttonImgA0" src="img/start_a.png" width="100%" height="100%"><img id="buttonImgB0" src="img/start_b.png" width="100%" height="100%"></div>'; // insert html
    document.getElementById('buttonImgA0').style.position      = 'absolute';
    document.getElementById('buttonImgB0').style.position      = 'absolute';
    document.getElementById('buttonImgB0').style.opacity       = 0;
    document.getElementById('buttonImgA0').style.transition    = "all 1s";
    document.getElementById('buttonImgB0').style.transition    = "all 1s";
    // generate dot buttons based on array length
    for (var i=1;i<buttonX.length;i++){
      interface.innerHTML += '<div id="dot'+i+'" onclick="buttonClicked('+i+')"><img id="buttonImgA'+i+'" src="img/pfeil_a.png" width="100%" height="100%"><img id="buttonImgB'+i+'" src="img/pfeil_b.png" width="100%" height="100%"></div>'; // insert html

      var dotButtonA = document.getElementById('buttonImgA'+i);
      dotButtonA.style.position   = 'absolute';
      dotButtonA.style.transition = "all 1s";


      var dotButtonB = document.getElementById('buttonImgB'+i);
      dotButtonA.style.position   = 'absolute';
      dotButtonB.style.transition = "all 1s";
    }
  }
  setButtonPosition();
  buttonsAreGenerated = 1;
}

//============================================================== SET DOT BUTTON POSITION
function setButtonPosition(){

  // start button
  var startButtonRatio  = 1040/2560;
  var startButtonWidth  = buttonSize * 1.8;
  var startButtonHeight = buttonSize  * 1.8 * startButtonRatio;
  var startButton = document.getElementById('dot0');
  if (startButton){
    startButton.style.left            = (w-startButtonWidth)/2 +'px';
    startButton.style.bottom          = Math.min((w-startButtonWidth)/2, (h/5))+'px';
    startButton.style.width           = startButtonWidth +'px';
    startButton.style.height          = startButtonHeight +'px';
    startButton.style.position        = 'fixed';
    startButton.style.display         = 'inline-block';
    startButton.style.zIndex          =  15;
    startButton.style.cursor          = 'pointer';
    startButton.style.visibility      = 'visible';
  }

  // dot buttons with parallax effect
  var xParallax = 0
  if (imgWidth>w){
    xParallax = parallax * ((window.scrollX/(imgWidth-w))-.5)
  };
  var yParallax = 0;
  if (imgHeight>h){
    yParallax = parallax * ((window.scrollY/(imgHeight-h))-.5);
  }

  for (var i=1;i<buttonX.length;i++){
    button[i] = document.getElementById('dot'+i);             // get id of the generated button
    if(button[i] && button[i].style) {                        // set position and style of the button
        button[i].style.left            = (imgWidth  * (buttonX[i] + xParallax) - buttonSize/2)  +'px';
        button[i].style.top             = (imgHeight * (buttonY[i] + yParallax) - buttonSize/2)  +'px';
        button[i].style.width           = buttonSize +'px';
        button[i].style.height          = buttonSize +'px';
        button[i].style.position        = 'absolute';
        button[i].style.display         = 'inline-block';
        button[i].style.zIndex          = i+10;
        button[i].style.cursor          = 'pointer';
        button[i].style.visibility      = 'visible';
        button[i].style.borderRadius    = '50%';
    }
  }

}
//============================================================== SET DOT BUTTON COLORS // DU_ROCKST!
function setButtonColors(){
  console.log('set button colors');

  //------------------------------------------------------------ start button
  var startButton = document.getElementById('dot0');              // get id of the start button
  if(startButton && startButton.style) {                          // set position and style of the button
    if (buttonState[0] == 1){                                     // initial color based on state
    document.getElementById('buttonImgA0').style.transition    = "all .2s";
    document.getElementById('buttonImgB0').style.transition    = "all .2s";
    document.getElementById('buttonImgA0').style.opacity      = 0; // state = 1: blue
    document.getElementById('buttonImgB0').style.opacity      = 1;


      startResetTime = timeCount+500;                             // set time to reset start button
    }else{
    document.getElementById('buttonImgA0').style.transition    = "all 5s";
    document.getElementById('buttonImgB0').style.transition    = "all 5s";
      document.getElementById('buttonImgA0').style.opacity      = 1;    // state = -1: red
      document.getElementById('buttonImgB0').style.opacity      = 0;
    }
  }
    //------------------------------------------------------------ dot buttons
  var interface = document.getElementById('interface');
  for ( var i=1; i<buttonX.length; i++ ){                         // iterate through all dot buttons
    button[i] = document.getElementById('dot' + i);               // get id of the buttons
    if(button[i] && button[i].style) {                            // set position and style of the button
      if (buttonState[i] == 1){                                   // initial color based on state
        var dotButtonA = document.getElementById('buttonImgA'+i);
        var dotButtonB = document.getElementById('buttonImgB'+i);
        dotButtonA.style.opacity = 0;
        dotButtonB.style.opacity = 1;
        dotButtonA.style.transform = "rotate(-90deg)";
        dotButtonB.style.transform = "rotate(0deg)";
      }else{
        var dotButtonA = document.getElementById('buttonImgA'+i);
        var dotButtonB = document.getElementById('buttonImgB'+i);
        dotButtonA.style.opacity = 1;
        dotButtonB.style.opacity = 0;
        dotButtonA.style.transform = "rotate(0deg)";
        dotButtonB.style.transform = "rotate(90deg)";
      }
    }
  }
}

//============================================================== INTERACTION / BUTTONS
function buttonClicked(buttonIndex){         
  console.log(buttonIndex, buttonState[buttonIndex]);                 // function is called when a button is clicked
  if (buttonIndex>0 || (buttonIndex==0 && buttonState[0]==-1)){ // toggle only round dot buttons or start button if it is red
    var newButtonState = buttonState[buttonIndex]*-1;       // toggle button state
    sendButton(buttonIndex,newButtonState);
    //buttonID = document.getElementById('dot'+buttonIndex);      // get button ID
  }

  //setButtonColors();
}

//============================================================== GET POSITON
var mouseX = 0;
var mouseY = 0;
function getPosition(event) {
  mouseX = (event.clientX+ window.scrollX)/imgWidth; // mouse position on bgnd (0...1)
  mouseY = (event.clientY + window.scrollY)/imgHeight;
  setButtonPosition(); // parallax scrolling 
}

//============================================================== SHOW DEBUG INFO
function showDebug() {
  if (debug==1){
    var coords = "input coordinates x: " + mouseX.toFixed(3) + "  y: " + mouseY.toFixed(3);
    var buttonStates ='';
    for (var i=0;i<buttonState.length;i++){
      buttonStates+='['+(buttonState[i]+1)/2+']  ';
    }
    var scrollInfo = 'scroll  x: '+ window.scrollX.toFixed(3) +'  y: '+ window.scrollY.toFixed(3);

    var wSize =      'screen  x: '+window.innerWidth+'  y: '+window.innerHeight;
    document.getElementById("infoBox").style.visibility ='visible';
    document.getElementById("infoBox").innerHTML = coords + '<br> ' + buttonStates + '<br> ' + wSize +  '<br> ' + scrollInfo + '<br>time: '+timeCount;
  }else{
    document.getElementById("infoBox").style.visibility ='hidden';
  }
}

//============================================================== SEND BUTTON
function sendButton(sendButtonIndex, sendButtonState ){
  var data = {
      index: sendButtonIndex,
      state: sendButtonState
  }
  socket.emit('buttonMessage',data);
}

//============================================================== RECEIVE BUTTON
function receiveButton(data){
  buttonState[data.index] = data.state;
  setButtonColors();

}